-- *****************************************************************************
-- *
-- * Licensed Materials - Property of IBM
-- * 5724-C40
-- * (C) Copyright IBM Corp. 2002.  All Rights Reserved.
-- *		
-- * US Government Users Restricted Rights - Use, duplication or
-- * disclosure restricted by GSA ADP Schedule Contract with
-- * IBM Corp.
-- *
-- *****************************************************************************

-----------------------------------------------------------------------------------
-- Here we perform an incremental extract using date/time to control the extraction
-- window because we could be interested in old DM data; i.e. to DM data  retrieved
-- before the TWH  patch that  introduce the  raw/integer  sequence numbers to keep 
-- track of every record added to the DM_METRICS table.
-- The source table is DM_METRICS and is located on an DB2 RIM Database 
-- (ODBC Src=db2_rim); the target table is AMW.STAGE_DM_METRICS and is located on a 
-- target DB2 Database (ODBC Tgt=twh_cdw).
-----------------------------------------------------------------------------------

--
-- //////////////// here we define the oldest dt_stamp to be considered during the 
-- //////////////// fist time ETL as the current timestamp minus the number of days
-- //////////////// specified in the PMSMTC_AGE_IN_DAYS field of the Prune_Msmt_Control
-- //////////////// table for the AMW application.
-- //////////////// You have to update the Prune_Msmt_Control table before running
-- //////////////// this initial ETL; e.g. if you are interested into the latest
-- //////////////// 2 months and 8 days of DM Data, the SQL statement is:
-- ////////////////
-- ////////////////     UPDATE TWG.Prune_Msmt_Control
-- ////////////////        SET PMSMTC_AGE_IN_DAYS = 208
-- ////////////////      WHERE TMSUM_CD = 'H'
-- ////////////////        AND MSRC_CD  = 'AMW'
-- ////////////////     ;
-- ////////////////
-- //////////////// because PMSMTC_AGE_IN_DAYS is a DECIMAL(8,0) field that holds a 
-- //////////////// date duration in the format yyyymmdd (208 -> yyyy=0 years, mm=2
-- //////////////// months, dd=8 days).
-- //////////////// The initial update of the Extract_Control table at the CDW database
-- //////////////// is performed only if the extraction window is open, i.e. if we are 
-- //////////////// running the initial ETL for the first time; this condition is intended
-- //////////////// in order to prevent multiple runs of the initialization script, that
-- //////////////// may lead to data integrity violation in the TWG.Msmt table.
--

--#EXECUTE_AT_TARGET
UPDATE TWG.Extract_Control 
  SET extctl_from_intseq = (
      SELECT (DAYS(current date - C.PMSMTC_AGE_IN_DAYS ) - DAYS('1970-01-01')) * 86400
        FROM TWG.PRUNE_MSMT_CONTROL C
       WHERE TMSUM_CD = 'H' AND 
             MSRC_CD  = 'AMW' )
WHERE
  extctl_source='DM_METRICS_INIT' AND
  extctl_target='AMW.STAGE_DM_METRICS' AND
  extctl_from_intseq <> extctl_to_intseq
;

--
-- //////////////// drop temporary table to hold the extract window
--
-- //////////////// at: db2_rim
--
--#IGNORE_ERROR 
DROP TABLE AMW.temp_extract_control
;
--
-- //////////////// create temporary table to hold the extract window
--
-- //////////////// at: db2_rim
--
CREATE TABLE AMW.temp_extract_control (
  extctl_from_intseq   INTEGER NOT NULL,
  extctl_to_intseq     INTEGER NOT NULL )
;
--
-- //////////////// get the from_intseq from the Extract_Control in the CDW
--
-- //////////////// from: twh_cdw to: db2_rim
--
--#INSERT_INTO_SOURCE
INSERT INTO AMW.temp_extract_control
SELECT
  INT(extctl_from_intseq),
  INT(extctl_to_intseq)
FROM 
  TWG.Extract_Control
WHERE
  extctl_source='DM_METRICS_INIT' AND
  extctl_target='AMW.STAGE_DM_METRICS'
;
-- //////////////// update extctl_to_intseq with the max of dt_stamp
--
-- //////////////// at: db2_rim
--
UPDATE AMW.temp_extract_control
   SET extctl_to_intseq = (SELECT MAX(dt_stamp) FROM DM_METRICS)
 WHERE extctl_from_intseq <> extctl_to_intseq
;
-- //////////////// drop the staging table in twh_cdw (this is faster then deleting all records)
--
-- //////////////// at: twh_cdw
--
--#EXECUTE_AT_TARGET
--#IGNORE_ERROR          
DROP TABLE AMW.stage_dm_metrics
;
-- //////////////// create the stage table in twh_cdw according to the predefined template
--
-- //////////////// at: twh_cdw
--
--#EXECUTE_AT_TARGET
CREATE TABLE AMW.stage_dm_metrics LIKE AMW.stage_dm_metrics_template
;
-- //////////////// Dropping the table AMW.stage_dm_metrics makes inoperative the  
-- //////////////// trigger responsible to insert the metric_id unique sequence number.
-- //////////////// Issuing a CREATE TRIGGER statement with the same trigger-name 
-- //////////////// as the inoperative trigger will cause that inoperative trigger 
-- //////////////// to be replaced with a warning (SQLSTATE 01595), so we need to
-- //////////////// instruct the execsql to IGNORE_ERROR.
--
--#EXECUTE_AT_TARGET
--#IGNORE_ERROR
CREATE TRIGGER AMW.metric_id_trig NO CASCADE 
  BEFORE INSERT ON AMW.stage_dm_metrics  
  REFERENCING  NEW AS N  
  FOR EACH ROW 
  MODE DB2SQL 
  SET N.metric_id = NEXTVAL FOR AMW.metric_id_seq
;
--
-- //////////////// perform the extraction from the DM_METRICS table to the staging
-- //////////////// table AMW.stage_dm_metrics in the CDW database.
--
-- //////////////// from: db2_rim to: twh_cdw
--
-- //////////////// NOTE: the constant metric_id value selected from the DM_METRICS table
-- //////////////// is always overridden by the unique sequence number provided by the 
-- //////////////// before insert trigger AMW.metric_id_trig
--
-- //////////////// The initial ETL process assumes the local GMT offset;
-- //////////////// if you want to change it, you can change the last zero constant
-- //////////////// field retrieved by the following select to a more suitable value
-- //////////////// to be chosen in the range [-720,+720].
--
--#INSERT_INTO_TARGET
INSERT INTO AMW.stage_dm_metrics 
SELECT
  0 AS metric_id,
  D.COLLECTION_DATE, D.HOSTNAME, D.ENDPOINT, 
  D.PROFILE_COLLECTION, D.PROBE_COLLECTION, 
  D.PROBE, D.PROBE_DESC, D.PROBE_ARG, 
  D.MIN_VALUE_00, D.MAX_VALUE_00, D.AVG_VALUE_00, 
  D.MIN_VALUE_01, D.MAX_VALUE_01, D.AVG_VALUE_01,
  D.MIN_VALUE_02, D.MAX_VALUE_02, D.AVG_VALUE_02,
  D.MIN_VALUE_03, D.MAX_VALUE_03, D.AVG_VALUE_03,
  D.MIN_VALUE_04, D.MAX_VALUE_04, D.AVG_VALUE_04,
  D.MIN_VALUE_05, D.MAX_VALUE_05, D.AVG_VALUE_05,
  D.MIN_VALUE_06, D.MAX_VALUE_06, D.AVG_VALUE_06,
  D.MIN_VALUE_07, D.MAX_VALUE_07, D.AVG_VALUE_07,
  D.MIN_VALUE_08, D.MAX_VALUE_08, D.AVG_VALUE_08,
  D.MIN_VALUE_09, D.MAX_VALUE_09, D.AVG_VALUE_09,
  D.MIN_VALUE_10, D.MAX_VALUE_10, D.AVG_VALUE_10,
  D.MIN_VALUE_11, D.MAX_VALUE_11, D.AVG_VALUE_11,
  D.MIN_VALUE_12, D.MAX_VALUE_12, D.AVG_VALUE_12,
  D.MIN_VALUE_13, D.MAX_VALUE_13, D.AVG_VALUE_13,
  D.MIN_VALUE_14, D.MAX_VALUE_14, D.AVG_VALUE_14,
  D.MIN_VALUE_15, D.MAX_VALUE_15, D.AVG_VALUE_15,
  D.MIN_VALUE_16, D.MAX_VALUE_16, D.AVG_VALUE_16,
  D.MIN_VALUE_17, D.MAX_VALUE_17, D.AVG_VALUE_17,
  D.MIN_VALUE_18, D.MAX_VALUE_18, D.AVG_VALUE_18,
  D.MIN_VALUE_19, D.MAX_VALUE_19, D.AVG_VALUE_19,
  D.MIN_VALUE_20, D.MAX_VALUE_20, D.AVG_VALUE_20,
  D.MIN_VALUE_21, D.MAX_VALUE_21, D.AVG_VALUE_21,
  D.MIN_VALUE_22, D.MAX_VALUE_22, D.AVG_VALUE_22,
  D.MIN_VALUE_23, D.MAX_VALUE_23, D.AVG_VALUE_23,
  D.MIN_DAILY_VALUE, D.MAX_DAILY_VALUE, D.AVG_DAILY_VALUE,
  D.MIN_HOURLY_AVG, D.MAX_HOURLY_AVG,
  (current timezone * 60 / 10000)
FROM 
  DM_METRICS D,
  AMW.temp_extract_control ec
WHERE
  D.DT_STAMP > ec.extctl_from_intseq AND
  D.DT_STAMP <= ec.extctl_to_intseq
;
--
-- //////////////// sample check for invalid data if multi customer support is implemented
--
-- //////////////// at: twh_cdw 
--
--#EXECUTE_AT_TARGET
INSERT INTO AMW.Invalid_Data
SELECT *
  FROM AMW.stage_dm_metrics N
 WHERE NOT EXISTS ( SELECT 1 
                      FROM AMW.CUST_LOOKUP CUST_LOOKUP
                     WHERE CUST_LOOKUP.VALUE = N.HOSTNAME OR 
                           CUST_LOOKUP.VALUE = '@' ) 
       OR 
       NOT EXISTS ( SELECT 1 
                      FROM AMW.CENTR_LOOKUP CENTR_LOOKUP
                     WHERE CENTR_LOOKUP.VALUE = N.HOSTNAME OR 
                           CENTR_LOOKUP.VALUE = '@' )
;
--
-- //////////////// drop temp_to_intseq temporary table in CDW
--
-- //////////////// at: twh_cdw 
--
--#EXECUTE_AT_TARGET
--#IGNORE_ERROR          
DROP TABLE AMW.temp_to_intseq
;
-- //////////////// create temp_to_intseq temporary table in CDW
--
-- //////////////// at: twh_cdw
--
--#EXECUTE_AT_TARGET
CREATE TABLE AMW.temp_to_intseq (
  extctl_to_intseq   INTEGER NOT NULL ) 
;
-- //////////////// copy extctl_to_intseq from temp_extract_control table 
-- //////////////// temp_to_intseq temporary table in the CDW
--
-- //////////////// from: db2_rim to: twh_cdw
--
--#INSERT_INTO_TARGET
INSERT INTO AMW.temp_to_intseq
  SELECT extctl_to_intseq FROM AMW.temp_extract_control
;
-- //////////////// update extract control with the extctl_to_intseq value 
-- //////////////// retrieved from the temporary table temp_to_intseq
--
-- //////////////// at: twh_cdw
--
--#EXECUTE_AT_TARGET
UPDATE TWG.Extract_Control
  SET extctl_to_intseq = (SELECT extctl_to_intseq FROM AMW.temp_to_intseq)
WHERE
  extctl_source='DM_METRICS_INIT' AND
  extctl_target='AMW.STAGE_DM_METRICS'
;
-- //////////////// write extract info to extract log table; this fires the trigger
-- //////////////// TWG.extract_ctl_upd that updates the extract control table to 
-- //////////////// close the window
--
-- //////////////// at: twh_cdw
-- 
--#EXECUTE_AT_TARGET
INSERT INTO TWG.Extract_Log
SELECT 
  extctl_source, extctl_target,
  current timestamp - current timezone,
  extctl_from_rawseq, extctl_to_rawseq,
  extctl_from_intseq, extctl_to_intseq,
  extctl_from_dttm, extctl_to_dttm
FROM
  TWG.Extract_Control ec
WHERE
  ec.extctl_source='DM_METRICS_INIT' AND
  ec.extctl_target='AMW.STAGE_DM_METRICS' 
;

